'use strict';

const del = require('del');
const gulp = require('gulp');
const gutil = require('gulp-util');
const env = require('gulp-env');
const webpack = require('webpack');
const shell = require('shelljs');
const config = require('./config');

// clean task
const cleanTask = (files) => () => {
  del(files, (err, paths) => {
    gutil.log('Deleted files/folders:\n', gutil.colors.cyan(paths.join('\n')));
  });
};

gulp.task('clean', cleanTask([
  `.${config.path.build}`,
  `.${config.path.publicAssets}`,
]));

gulp.task('set-production-env', () => {
  env({
    vars: {
      RUN_MODE: 'es',
      NODE_ENV: 'production',
    },
  });
});

gulp.task('set-development-env', () => {
  env({
    vars: {
      RUN_MODE: 'es',
    },
  });
});

gulp.task('compile-templates', (done) =>
  shell.exec('yarn run compile-templates', () => done())
);

gulp.task('update-schema', (done) =>
  shell.exec('yarn run update-schema', () => done())
);

gulp.task('frontend:watch', ['set-development-env'], (done) =>
  shell.exec('yarn run frontend:watch', () => done())
);

gulp.task('frontend:build', ['set-production-env'], (done) => {
  webpack(require('./config/webpack/client/production'))
    .run((err, stats) => {
      if (err) {
        throw new gutil.PluginError('webpack', err);
      }

      gutil.log('[webpack]', stats.toString({ colors: true }));
      done();
    });
});

gulp.task('backend:watch', ['set-development-env'], (done) => {
  let started = false;

  webpack(require('./config/webpack/server/development'))
    .watch(100, (err, stats) => {
      if (!started) {
        started = true;
        done();
      }

      gutil.log('[webpack]', stats.toString({ colors: true }));
    });
});

gulp.task('backend:build', ['set-production-env'], (done) => {
  webpack(require('./config/webpack/server/production'))
    .run((err, stats) => {
      if (err) {
        throw new gutil.PluginError('webpack', err);
      }

      gutil.log('[webpack]', stats.toString({ colors: true }));
      done();
    });
});

gulp.task('build', ['clean', 'compile-templates', 'update-schema', 'frontend:build', 'backend:build']);
